@@ -1,18 +1,17 @@
 #!/usr/bin/env python
-# vim:fileencoding=UTF-8:ts=4:sw=4:sta:et:sts=4:ai
 
 
 __license__   = 'GPL v3'
 __copyright__ = '2011, Kovid Goyal <kovid@kovidgoyal.net>'
 __docformat__ = 'restructuredtext en'
 
-import numbers
-from struct import pack
 import io
+import numbers
 from collections import OrderedDict, defaultdict
+from struct import pack
 
-from calibre.ebooks.mobi.utils import (encint, encode_number_as_hex,
-        encode_tbs, align_block, RECORD_SIZE, CNCX as CNCX_)
+from calibre.ebooks.mobi.utils import CNCX as CNCX_
+from calibre.ebooks.mobi.utils import RECORD_SIZE, align_block, encint, encode_number_as_hex, encode_tbs
 from polyglot.builtins import iteritems, itervalues
 
 
@@ -38,7 +37,7 @@ class TAGX:  # {{{
     BITMASKS.update({x:(1 << i) for i, x in enumerate([1, 2, 3, 4, 5, 21, 22, 23])})
     BITMASKS.update({x:(1 << i) for i, x in enumerate([69, 70, 71, 72, 73])})
 
-    NUM_VALUES = defaultdict(lambda :1)
+    NUM_VALUES = defaultdict(lambda:1)
     NUM_VALUES[11] = 3
     NUM_VALUES[0] = 0
 
@@ -65,8 +64,8 @@ class TAGX:  # {{{
         '''
         TAGX block for the Primary index header of a periodical
         '''
-        list(map(self.add_tag, (1, 2, 3, 4, 5, 21, 22, 23, 0, 69, 70, 71, 72,
-            73, 0)))
+        for i in (1, 2, 3, 4, 5, 21, 22, 23, 0, 69, 70, 71, 72, 73, 0):
+            self.add_tag(i)
         return self.header(2) + bytes(self.byts)
 
     @property
@@ -74,7 +73,8 @@ class TAGX:  # {{{
         '''
         TAGX block for the secondary index header of a periodical
         '''
-        list(map(self.add_tag, (11, 0)))
+        for i in (11, 0):
+            self.add_tag(i)
         return self.header(1) + bytes(self.byts)
 
     @property
@@ -82,12 +82,13 @@ class TAGX:  # {{{
         '''
         TAGX block for the primary index header of a flat book
         '''
-        list(map(self.add_tag, (1, 2, 3, 4, 0)))
+        for i in (1, 2, 3, 4, 0):
+            self.add_tag(i)
         return self.header(1) + bytes(self.byts)
 
-
 # }}}
 
+
 # Index Entries {{{
 
 class IndexEntry:
@@ -107,7 +108,7 @@ class IndexEntry:
             'author_offset': 71,
 
     }
-    RTAG_MAP = {v:k for k, v in iteritems(TAG_VALUES)}  # noqa
+    RTAG_MAP = {v:k for k, v in iteritems(TAG_VALUES)}
 
     def __init__(self, offset, label_offset):
         self.offset, self.label_offset = offset, label_offset
@@ -126,9 +127,8 @@ class IndexEntry:
         self.desc_offset = None
 
     def __repr__(self):
-        return ('IndexEntry(offset=%r, depth=%r, length=%r, index=%r,'
-                ' parent_index=%r)')%(self.offset, self.depth, self.length,
-                        self.index, self.parent_index)
+        return (f'IndexEntry(offset={self.offset!r}, depth={self.depth!r}, length={self.length!r}, index={self.index!r},'
+                f' parent_index={self.parent_index!r})')
 
     @property
     def size(self):
@@ -221,7 +221,7 @@ class SecondaryIndexEntry(IndexEntry):
         tag = self.INDEX_MAP[index]
 
         # The values for this index entry
-        # I dont know what the 5 means, it is not the number of entries
+        # I don't know what the 5 means, it is not the number of entries
         self.secondary = [5 if tag == min(
             itervalues(self.INDEX_MAP)) else 0, 0, tag]
 
@@ -368,7 +368,7 @@ class TBS:  # {{{
 
                 try:
                     next_sec = sections[i+1]
-                except:
+                except Exception:
                     next_sec = None
 
                 extra = {}
@@ -447,8 +447,7 @@ class Indexer:  # {{{
         if self.is_periodical and self.masthead_offset is None:
             raise ValueError('Periodicals must have a masthead')
 
-        self.log('Generating MOBI index for a %s'%('periodical' if
-            self.is_periodical else 'book'))
+        self.log('Generating MOBI index for a {}'.format('periodical' if self.is_periodical else 'book'))
         self.is_flat_periodical = False
         if self.is_periodical:
             periodical_node = next(iter(oeb.toc))
@@ -530,7 +529,7 @@ class Indexer:  # {{{
 
         ans = header + body
         if len(ans) > 0x10000:
-            raise ValueError('Too many entries (%d) in the TOC'%len(offsets))
+            raise ValueError(f'Too many entries ({len(offsets)}) in the TOC')
         return ans
     # }}}
 
@@ -636,9 +635,8 @@ class Indexer:  # {{{
             try:
                 offset = id_offsets[node.href]
                 label = self.cncx[node.title]
-            except:
-                self.log.warn('TOC item %s [%s] not found in document'%(
-                    node.title, node.href))
+            except Exception:
+                self.log.warn(f'TOC item {node.title} [{node.href}] not found in document')
                 continue
 
             if offset in seen:
@@ -653,7 +651,7 @@ class Indexer:  # {{{
         for i, index in enumerate(indices):
             try:
                 next_offset = indices[i+1].offset
-            except:
+            except Exception:
                 next_offset = self.serializer.body_end_offset
             index.length = next_offset - index.offset
 
@@ -664,7 +662,7 @@ class Indexer:  # {{{
         for i, index in enumerate(indices):
             try:
                 next_offset = indices[i+1].offset
-            except:
+            except Exception:
                 next_offset = self.serializer.body_end_offset
             index.length = next_offset - index.offset
 
@@ -702,7 +700,7 @@ class Indexer:  # {{{
                 offset = id_offsets[sec.href]
                 label = self.cncx[sec.title]
                 klass = self.cncx[sec.klass]
-            except:
+            except Exception:
                 continue
             if offset in seen_sec_offsets:
                 continue
@@ -716,7 +714,7 @@ class Indexer:  # {{{
                     offset = id_offsets[art.href]
                     label = self.cncx[art.title]
                     klass = self.cncx[art.klass]
-                except:
+                except Exception:
                     continue
                 if offset in seen_art_offsets:
                     continue
@@ -744,12 +742,12 @@ class Indexer:  # {{{
             sec, normalized_articles = x
             try:
                 sec.length = normalized_sections[s+1][0].offset - sec.offset
-            except:
+            except Exception:
                 sec.length = self.serializer.body_end_offset - sec.offset
             for i, art in enumerate(normalized_articles):
                 try:
                     art.length = normalized_articles[i+1].offset - art.offset
-                except:
+                except Exception:
                     art.length = sec.offset + sec.length - art.offset
 
         # Filter
@@ -785,14 +783,14 @@ class Indexer:  # {{{
             sec, articles = x
             try:
                 next_offset = normalized_sections[s+1][0].offset
-            except:
+            except Exception:
                 next_offset = self.serializer.body_end_offset
             sec.length = next_offset - sec.offset
 
             for a, art in enumerate(articles):
                 try:
                     next_offset = articles[a+1].offset
-                except:
+                except Exception:
                     next_offset = sec.next_offset
                 art.length = next_offset - art.offset
 
@@ -801,7 +799,7 @@ class Indexer:  # {{{
             sec, articles = x
             try:
                 next_sec = normalized_sections[s+1][0]
-            except:
+            except Exception:
                 if (sec.length == 0 or sec.next_offset !=
                         self.serializer.body_end_offset):
                     raise ValueError('Invalid section layout')
@@ -811,7 +809,7 @@ class Indexer:  # {{{
             for a, art in enumerate(articles):
                 try:
                     next_art = articles[a+1]
-                except:
+                except Exception:
                     if (art.length == 0 or art.next_offset !=
                             sec.next_offset):
                         raise ValueError('Invalid article layout')
