@@ -66,7 +66,7 @@ FONT_SIZE_NAMES = {
 }
 
 ALLOWED_MEDIA_TYPES = frozenset({'screen', 'all', 'aural', 'amzn-kf8'})
-IGNORED_MEDIA_FEATURES = frozenset('width min-width max-width height min-height max-height device-width min-device-width max-device-width device-height min-device-height max-device-height aspect-ratio min-aspect-ratio max-aspect-ratio device-aspect-ratio min-device-aspect-ratio max-device-aspect-ratio color min-color max-color color-index min-color-index max-color-index monochrome min-monochrome max-monochrome -webkit-min-device-pixel-ratio resolution min-resolution max-resolution scan grid'.split())  # noqa
+IGNORED_MEDIA_FEATURES = frozenset('width min-width max-width height min-height max-height device-width min-device-width max-device-width device-height min-device-height max-device-height aspect-ratio min-aspect-ratio max-aspect-ratio device-aspect-ratio min-device-aspect-ratio max-device-aspect-ratio color min-color max-color color-index min-color-index max-color-index monochrome min-monochrome max-monochrome -webkit-min-device-pixel-ratio resolution min-resolution max-resolution scan grid'.split())  # noqa: E501
 
 
 def media_ok(raw):
@@ -86,7 +86,7 @@ def media_ok(raw):
         return mq.negated ^ matched
 
     try:
-        for mq in CSSMedia3Parser().parse_stylesheet('@media %s {}' % raw).rules[0].media:
+        for mq in CSSMedia3Parser().parse_stylesheet(f'@media {raw} {{}}').rules[0].media:
             if query_ok(mq):
                 return True
         return False
@@ -184,7 +184,7 @@ class StylizerRules:
             if size == 'smallest':
                 size = 'xx-small'
             if size in FONT_SIZE_NAMES:
-                style['font-size'] = "%.1frem" % (self.profile.fnames[size] / float(self.profile.fbase))
+                style['font-size'] = f'{self.profile.fnames[size]/float(self.profile.fbase):.1f}rem'
         if '-epub-writing-mode' in style:
             for x in ('-webkit-writing-mode', 'writing-mode'):
                 style[x] = style.get(x, style['-epub-writing-mode'])
@@ -274,7 +274,7 @@ class Stylizer:
                                 continue
                             sitem = hrefs[ihref]
                             if sitem.media_type not in OEB_STYLES:
-                                self.logger.warn('CSS @import of non-CSS file %r' % rule.href)
+                                self.logger.warn(f'CSS @import of non-CSS file {rule.href!r}')
                                 continue
                             stylesheets.append(sitem.data)
                     # Make links to resources absolute, since these rules will
@@ -291,13 +291,11 @@ class Stylizer:
                 sitem = oeb.manifest.hrefs.get(path, None)
                 if sitem is None:
                     self.logger.warn(
-                        'Stylesheet %r referenced by file %r not in manifest' %
-                        (path, item.href))
+                        f'Stylesheet {path!r} referenced by file {item.href!r} not in manifest')
                     continue
                 if not hasattr(sitem.data, 'cssRules'):
                     self.logger.warn(
-                    'Stylesheet %r referenced by file %r is not CSS'%(path,
-                        item.href))
+                    f'Stylesheet {path!r} referenced by file {item.href!r} is not CSS')
                     continue
                 stylesheets.append(sitem.data)
         csses = {'extra_css':extra_css, 'user_css':user_css}
@@ -309,7 +307,7 @@ class Stylizer:
                             validate=False)
                     stylesheets.append(stylesheet)
                 except Exception:
-                    self.logger.exception('Failed to parse %s, ignoring.'%w)
+                    self.logger.exception(f'Failed to parse {w}, ignoring.')
                     self.logger.debug('Bad css: ')
                     self.logger.debug(x)
 
@@ -324,7 +322,7 @@ class Stylizer:
         self.flatten_style = self.oeb.stylizer_rules.flatten_style
 
         self._styles = {}
-        pseudo_pat = re.compile(':{1,2}(%s)' % ('|'.join(INAPPROPRIATE_PSEUDO_CLASSES)), re.I)
+        pseudo_pat = re.compile(':{{1,2}}({})'.format('|'.join(INAPPROPRIATE_PSEUDO_CLASSES)), re.I)
         select = Select(tree, ignore_inappropriate_pseudo_classes=True)
 
         for _, _, cssdict, text, _ in self.rules:
@@ -354,7 +352,7 @@ class Stylizer:
 
                                 special_text = ''.join(punctuation_chars) + \
                                         (text[0] if text else '')
-                                span = x.makeelement('{%s}span' % XHTML_NS)
+                                span = x.makeelement(f'{{{XHTML_NS}}}span')
                                 span.text = special_text
                                 span.set('data-fake-first-letter', '1')
                                 span.tail = text[1:]
@@ -383,7 +381,7 @@ class Stylizer:
                     val = elem.get(prop, '').strip()
                     try:
                         del elem.attrib[prop]
-                    except:
+                    except Exception:
                         pass
                     if val:
                         if num_pat.match(val) is not None:
@@ -395,16 +393,16 @@ class Stylizer:
     def _fetch_css_file(self, path):
         hrefs = self.oeb.manifest.hrefs
         if path not in hrefs:
-            self.logger.warn('CSS import of missing file %r' % path)
-            return (None, None)
+            self.logger.warn(f'CSS import of missing file {path!r}')
+            return None, None
         item = hrefs[path]
         if item.media_type not in OEB_STYLES:
-            self.logger.warn('CSS import of non-CSS file %r' % path)
-            return (None, None)
+            self.logger.warn(f'CSS import of non-CSS file {path!r}')
+            return None, None
         data = item.data.cssText
         if not isinstance(data, bytes):
             data = data.encode('utf-8')
-        return ('utf-8', data)
+        return 'utf-8', data
 
     def style(self, element):
         try:
@@ -421,7 +419,7 @@ class Stylizer:
                     style['font-size'].endswith('pt'):
                 style = copy.copy(style)
                 size = float(style['font-size'][:-2])
-                style['font-size'] = "%.2fpt" % (size * font_scale)
+                style['font-size'] = f'{size*font_scale:.2f}pt'
             style = ';\n    '.join(': '.join(item) for item in style.items())
             rules.append(f'{selector} {{\n    {style};\n}}')
         return '\n'.join(rules)
@@ -438,6 +436,7 @@ def is_only_number(x: str) -> bool:
     except Exception:
         return False
 
+
 def is_svg_text_tag(x):
     return getattr(x, 'tag', '') in svg_text_tags
 
@@ -581,7 +580,7 @@ class Style:
                         val = style.getProperty('background').propertyValue
                         try:
                             val = list(val)
-                        except:
+                        except Exception:
                             # val is CSSPrimitiveValue
                             val = [val]
                         for c in val:
@@ -591,7 +590,7 @@ class Style:
                             if validate_color(c):
                                 col = c
                                 break
-                    except:
+                    except Exception:
                         pass
             if col is None:
                 self._bgcolor = False
@@ -628,7 +627,7 @@ class Style:
                 if not isinstance(result, numbers.Number):
                     return base
                 if result < 0:
-                    result = normalize_fontsize("smaller", base)
+                    result = normalize_fontsize('smaller', base)
             if factor:
                 result = factor * base
             return result
@@ -854,7 +853,7 @@ class Style:
 
     def __str__(self):
         items = sorted(iteritems(self._style))
-        return '; '.join(f"{key}: {val}" for key, val in items)
+        return '; '.join(f'{key}: {val}' for key, val in items)
 
     def cssdict(self):
         return dict(self._style)
