@@ -1,5 +1,4 @@
 #!/usr/bin/env python
-# vim:fileencoding=UTF-8:ts=4:sw=4:sta:et:sts=4:ai
 
 
 __license__   = 'GPL v3'
@@ -9,30 +8,53 @@ __docformat__ = 'restructuredtext en'
 import datetime
 import re
 import traceback
+
 from qt.core import (
-    QAbstractItemView, QAbstractTableModel, QAction, QApplication, QBrush, QComboBox,
-    QDialog, QDialogButtonBox, QFont, QFrame, QHBoxLayout, QIcon, QLabel, QLineEdit,
-    QModelIndex, QPixmap, QSize, QSortFilterProxyModel, Qt, QTableView, QUrl,
-    QVBoxLayout
+    QAbstractItemView,
+    QAbstractTableModel,
+    QAction,
+    QBrush,
+    QComboBox,
+    QDialog,
+    QDialogButtonBox,
+    QFont,
+    QFrame,
+    QHBoxLayout,
+    QIcon,
+    QLabel,
+    QLineEdit,
+    QModelIndex,
+    QSize,
+    QSortFilterProxyModel,
+    Qt,
+    QTableView,
+    QUrl,
+    QVBoxLayout,
 )
 
 from calibre import prints
-from calibre.constants import (
-    DEBUG, __appname__, __version__, ismacos, iswindows, numeric_version
-)
+from calibre.constants import DEBUG, __appname__, __version__, ismacos, iswindows, numeric_version
 from calibre.customize import PluginInstallationType
 from calibre.customize.ui import (
-    NameConflict, add_plugin, disable_plugin, enable_plugin, has_external_plugins,
-    initialized_plugins, is_disabled, remove_plugin
+    BLACKLISTED_PLUGINS,
+    NameConflict,
+    add_plugin,
+    disable_plugin,
+    enable_plugin,
+    has_external_plugins,
+    initialized_plugins,
+    is_disabled,
+    remove_plugin,
 )
 from calibre.gui2 import error_dialog, gprefs, info_dialog, open_url, question_dialog
 from calibre.gui2.preferences.plugins import ConfigWidget
 from calibre.utils.date import UNDEFINED_DATE, format_date
 from calibre.utils.https import get_https_resource_securely
+from calibre.utils.icu import lower as icu_lower
 from polyglot.builtins import itervalues
 
 SERVER = 'https://code.calibre-ebook.com/plugins/'
-INDEX_URL = '%splugins.json.bz2' % SERVER
+INDEX_URL = f'{SERVER}plugins.json.bz2'
 FILTER_ALL = 0
 FILTER_INSTALLED = 1
 FILTER_UPDATE_AVAILABLE = 2
@@ -74,17 +96,18 @@ def read_available_plugins(raise_error=False):
         if not raw:
             return
         raw = json.loads(bz2.decompress(raw))
-    except:
+    except Exception:
         if raise_error:
             raise
         traceback.print_exc()
         return
     for plugin in itervalues(raw):
         try:
-            display_plugin = DisplayPlugin(plugin)
-            get_installed_plugin_status(display_plugin)
-            display_plugins.append(display_plugin)
-        except:
+            if plugin['index_name'] not in BLACKLISTED_PLUGINS:
+                display_plugin = DisplayPlugin(plugin)
+                get_installed_plugin_status(display_plugin)
+                display_plugins.append(display_plugin)
+        except Exception:
             if DEBUG:
                 prints('======= Plugin Parse Error =======')
                 traceback.print_exc()
@@ -120,7 +143,6 @@ def get_installed_plugin_status(display_plugin):
 
 
 class ImageTitleLayout(QHBoxLayout):
-
     '''
     A reusable layout widget displaying an image followed by a title
     '''
@@ -130,13 +152,12 @@ class ImageTitleLayout(QHBoxLayout):
         title_font = QFont()
         title_font.setPointSize(16)
         title_image_label = QLabel(parent)
-        pixmap = QPixmap()
-        pixmap.load(I(icon_name))
-        if pixmap is None:
+        ic = QIcon.ic(icon_name)
+        if ic.isNull():
             error_dialog(parent, _('Restart required'),
                          _('You must restart calibre before using this plugin!'), show=True)
         else:
-            title_image_label.setPixmap(pixmap)
+            title_image_label.setPixmap(ic.pixmap(32, 32))
         title_image_label.setMaximumSize(32, 32)
         title_image_label.setScaledContents(True)
         self.addWidget(title_image_label)
@@ -147,7 +168,6 @@ class ImageTitleLayout(QHBoxLayout):
 
 
 class SizePersistedDialog(QDialog):
-
     '''
     This dialog is a base class for any dialogs that want their size/position
     restored when they are next opened.
@@ -158,18 +178,17 @@ class SizePersistedDialog(QDialog):
     def __init__(self, parent, unique_pref_name):
         QDialog.__init__(self, parent)
         self.unique_pref_name = unique_pref_name
-        self.geom = gprefs.get(unique_pref_name, None)
         self.finished.connect(self.dialog_closing)
 
+    def sizeHint(self):
+        ans = super().sizeHint()
+        return ans + self.initial_extra_size
+
     def resize_dialog(self):
-        if self.geom is None:
-            self.resize(self.sizeHint()+self.initial_extra_size)
-        else:
-            QApplication.instance().safe_restore_geometry(self, self.geom)
+        self.restore_geometry(gprefs, self.unique_pref_name)
 
     def dialog_closing(self, result):
-        geom = bytearray(self.saveGeometry())
-        gprefs[self.unique_pref_name] = geom
+        self.save_geometry(gprefs, self.unique_pref_name)
 
 
 class PluginFilterComboBox(QComboBox):
@@ -238,7 +257,7 @@ class DisplayPluginSortFilterModel(QSortFilterProxyModel):
         self.setSortRole(Qt.ItemDataRole.UserRole)
         self.setSortCaseSensitivity(Qt.CaseSensitivity.CaseInsensitive)
         self.filter_criteria = FILTER_ALL
-        self.filter_text = ""
+        self.filter_text = ''
 
     def filterAcceptsRow(self, sourceRow, sourceParent):
         index = self.sourceModel().index(sourceRow, 0, sourceParent)
@@ -314,7 +333,7 @@ class DisplayPluginModel(QAbstractTableModel):
                 return self._get_status_icon(display_plugin)
             if col == 1:
                 if display_plugin.donation_link:
-                    return QIcon(I('donate.png'))
+                    return QIcon.ic('donate.png')
         elif role == Qt.ItemDataRole.ToolTipRole:
             if col == 1 and display_plugin.donation_link:
                 return _('This plugin is FREE but you can reward the developer for their effort\n'
@@ -388,7 +407,7 @@ class DisplayPluginModel(QAbstractTableModel):
                 icon_name = 'plugin_new_valid.png'
             else:
                 icon_name = 'plugin_new_invalid.png'
-        return QIcon(I('plugins/' + icon_name))
+        return QIcon.ic('plugins/' + icon_name)
 
     def _get_status_tooltip(self, display_plugin):
         if display_plugin.is_deprecated:
@@ -416,6 +435,25 @@ class DisplayPluginModel(QAbstractTableModel):
                         _('Right-click to see more options'))
 
 
+def notify_on_successful_install(parent, plugin):
+    d = info_dialog(parent, _('Success'),
+            _('Plugin <b>{0}</b> successfully installed under <b>'
+                '{1}</b>. You may have to restart calibre '
+                'for the plugin to take effect.').format(plugin.name, plugin.type),
+            show_copy_button=False)
+    b = d.bb.addButton(_('&Restart calibre now'), QDialogButtonBox.ButtonRole.AcceptRole)
+    b.setIcon(QIcon.ic('lt.png'))
+    d.do_restart = False
+
+    def rf():
+        d.do_restart = True
+    b.clicked.connect(rf)
+    d.set_details('')
+    d.exec()
+    b.clicked.disconnect()
+    return d.do_restart
+
+
 class PluginUpdaterDialog(SizePersistedDialog):
 
     initial_extra_size = QSize(350, 100)
@@ -457,7 +495,7 @@ class PluginUpdaterDialog(SizePersistedDialog):
 
     def _initialize_controls(self):
         self.setWindowTitle(_('User plugins'))
-        self.setWindowIcon(QIcon(I('plugins/plugin_updater.png')))
+        self.setWindowIcon(QIcon.ic('plugins/plugin_updater.png'))
         layout = QVBoxLayout(self)
         self.setLayout(layout)
         title_layout = ImageTitleLayout(self, 'plugins/plugin_updater.png',
@@ -468,7 +506,7 @@ class PluginUpdaterDialog(SizePersistedDialog):
         layout.addLayout(header_layout)
         self.filter_combo = PluginFilterComboBox(self)
         self.filter_combo.setMinimumContentsLength(20)
-        self.filter_combo.currentIndexChanged[int].connect(self._filter_combo_changed)
+        self.filter_combo.currentIndexChanged.connect(self._filter_combo_changed)
         la = QLabel(_('Filter list of &plugins')+':', self)
         la.setBuddy(self.filter_combo)
         header_layout.addWidget(la)
@@ -480,7 +518,7 @@ class PluginUpdaterDialog(SizePersistedDialog):
         header_layout.addWidget(la)
         self.filter_by_name_lineedit = QLineEdit(self)
         la.setBuddy(self.filter_by_name_lineedit)
-        self.filter_by_name_lineedit.setText("")
+        self.filter_by_name_lineedit.setText('')
         self.filter_by_name_lineedit.textChanged.connect(self._filter_name_lineedit_changed)
 
         header_layout.addWidget(self.filter_by_name_lineedit)
@@ -525,17 +563,17 @@ class PluginUpdaterDialog(SizePersistedDialog):
     def update_forum_label(self):
         txt = ''
         if self.forum_link:
-            txt = '<a href="%s">%s</a>' % (self.forum_link, self.forum_label_text)
+            txt = f'<a href="{self.forum_link}">{self.forum_label_text}</a>'
         self.forum_label.setText(txt)
 
     def _create_context_menu(self):
         self.plugin_view.setContextMenuPolicy(Qt.ContextMenuPolicy.ActionsContextMenu)
-        self.install_action = QAction(QIcon(I('plugins/plugin_upgrade_ok.png')), _('&Install'), self)
+        self.install_action = QAction(QIcon.ic('plugins/plugin_upgrade_ok.png'), _('&Install'), self)
         self.install_action.setToolTip(_('Install the selected plugin'))
         self.install_action.triggered.connect(self._install_clicked)
         self.install_action.setEnabled(False)
         self.plugin_view.addAction(self.install_action)
-        self.forum_action = QAction(QIcon(I('plugins/mobileread.png')), _('Plugin &forum thread'), self)
+        self.forum_action = QAction(QIcon.ic('plugins/mobileread.png'), _('Plugin &forum thread'), self)
         self.forum_action.triggered.connect(self._forum_label_activated)
         self.forum_action.setEnabled(False)
         self.plugin_view.addAction(self.forum_action)
@@ -559,7 +597,7 @@ class PluginUpdaterDialog(SizePersistedDialog):
         sep2.setSeparator(True)
         self.plugin_view.addAction(sep2)
 
-        self.donate_enabled_action = QAction(QIcon(I('donate.png')), _('Donate to developer'), self)
+        self.donate_enabled_action = QAction(QIcon.ic('donate.png'), _('Donate to developer'), self)
         self.donate_enabled_action.setToolTip(_('Donate to the developer of this plugin'))
         self.donate_enabled_action.triggered.connect(self._donate_clicked)
         self.donate_enabled_action.setEnabled(False)
@@ -569,7 +607,7 @@ class PluginUpdaterDialog(SizePersistedDialog):
         sep3.setSeparator(True)
         self.plugin_view.addAction(sep3)
 
-        self.configure_action = QAction(QIcon(I('config.png')), _('&Customize plugin'), self)
+        self.configure_action = QAction(QIcon.ic('config.png'), _('&Customize plugin'), self)
         self.configure_action.setToolTip(_('Customize the options for this plugin'))
         self.configure_action.triggered.connect(self._configure_clicked)
         self.configure_action.setEnabled(False)
@@ -623,7 +661,7 @@ class PluginUpdaterDialog(SizePersistedDialog):
         self.plugin_view.setFocus()
 
     def _filter_combo_changed(self, idx):
-        self.filter_by_name_lineedit.setText("")  # clear the name filter text when a different group was selected
+        self.filter_by_name_lineedit.setText('')  # clear the name filter text when a different group was selected
         self.proxy_model.set_filter_criteria(idx)
         if idx == FILTER_NOT_INSTALLED:
             self.plugin_view.sortByColumn(5, Qt.SortOrder.DescendingOrder)
@@ -719,29 +757,13 @@ class PluginUpdaterDialog(SizePersistedDialog):
             widget.gui = self.gui
             widget.check_for_add_to_toolbars(plugin, previously_installed=plugin.name in installed_plugins)
             self.gui.status_bar.showMessage(_('Plugin installed: %s') % display_plugin.name)
-            d = info_dialog(self.gui, _('Success'),
-                    _('Plugin <b>{0}</b> successfully installed under <b>'
-                        '{1}</b>. You may have to restart calibre '
-                        'for the plugin to take effect.').format(plugin.name, plugin.type),
-                    show_copy_button=False)
-            b = d.bb.addButton(_('&Restart calibre now'), QDialogButtonBox.ButtonRole.AcceptRole)
-            b.setIcon(QIcon(I('lt.png')))
-            d.do_restart = False
-
-            def rf():
-                d.do_restart = True
-            b.clicked.connect(rf)
-            d.set_details('')
-            d.exec_()
-            b.clicked.disconnect()
-            do_restart = d.do_restart
-
+            do_restart = notify_on_successful_install(self.gui, plugin)
             display_plugin.plugin = plugin
             # We cannot read the 'actual' version information as the plugin will not be loaded yet
             display_plugin.installed_version = display_plugin.available_version
-        except:
+        except Exception:
             if DEBUG:
-                prints('ERROR occurred while installing plugin: %s'%display_plugin.name)
+                prints(f'ERROR occurred while installing plugin: {display_plugin.name}')
                 traceback.print_exc()
             error_dialog(self.gui, _('Install plugin failed'),
                          _('A problem occurred while installing this plugin.'
@@ -750,7 +772,7 @@ class PluginUpdaterDialog(SizePersistedDialog):
                            ' the forum thread for this plugin and restart calibre.'),
                          det_msg=traceback.format_exc(), show=True)
             if DEBUG:
-                prints('Due to error now uninstalling plugin: %s'%display_plugin.name)
+                prints(f'Due to error now uninstalling plugin: {display_plugin.name}')
             remove_plugin(display_plugin.name)
             display_plugin.plugin = None
 
@@ -783,7 +805,7 @@ class PluginUpdaterDialog(SizePersistedDialog):
         display_plugin = self._selected_display_plugin()
         plugin = display_plugin.plugin
         if not plugin.can_be_disabled:
-            return error_dialog(self,_('Plugin cannot be disabled'),
+            return error_dialog(self, _('Plugin cannot be disabled'),
                          _('The plugin: %s cannot be disabled')%plugin.name, show=True)
         if is_disabled(plugin):
             enable_plugin(plugin)
@@ -793,7 +815,7 @@ class PluginUpdaterDialog(SizePersistedDialog):
 
     def _download_zip(self, plugin_zip_url):
         from calibre.ptempfile import PersistentTemporaryFile
-        raw = get_https_resource_securely(plugin_zip_url, headers={'User-Agent':'%s %s' % (__appname__, __version__)})
+        raw = get_https_resource_securely(plugin_zip_url, headers={'User-Agent':f'{__appname__} {__version__}'})
         with PersistentTemporaryFile('.zip') as pt:
             pt.write(raw)
         return pt.name
