@@ -7,36 +7,58 @@ __docformat__ = 'restructuredtext en'
 
 import os
 import shutil
+
 from qt.core import (
-    QAbstractItemView, QDialog, QDialogButtonBox, QFont, QFontComboBox,
-    QFontDatabase, QFontInfo, QFontMetrics, QGridLayout, QHBoxLayout, QIcon, QLabel,
-    QLineEdit, QListView, QPen, QPushButton, QSize, QSizePolicy, QStringListModel,
-    QStyle, QStyledItemDelegate, Qt, QToolButton, QVBoxLayout, QWidget, pyqtSignal
+    QAbstractItemView,
+    QDialog,
+    QDialogButtonBox,
+    QFont,
+    QFontComboBox,
+    QFontDatabase,
+    QFontInfo,
+    QFontMetrics,
+    QGridLayout,
+    QHBoxLayout,
+    QIcon,
+    QLabel,
+    QLineEdit,
+    QListView,
+    QPen,
+    QPushButton,
+    QRawFont,
+    QSize,
+    QSizePolicy,
+    QStringListModel,
+    QStyle,
+    QStyledItemDelegate,
+    Qt,
+    QToolButton,
+    QVBoxLayout,
+    QWidget,
+    pyqtSignal,
 )
 
 from calibre.constants import config_dir
 from calibre.gui2 import choose_files, empty_index, error_dialog, info_dialog
+from calibre.utils.icu import lower as icu_lower
 
 
 def add_fonts(parent):
-    from calibre.utils.fonts.metadata import FontMetadata
     files = choose_files(parent, 'add fonts to calibre',
             _('Select font files'), filters=[(_('TrueType/OpenType Fonts'),
-                ['ttf', 'otf'])], all_files=False)
+                ['ttf', 'otf', 'woff', 'woff2'])], all_files=False)
     if not files:
         return
     families = set()
     for f in files:
-        try:
-            with open(f, 'rb') as stream:
-                fm = FontMetadata(stream)
-        except:
-            import traceback
+        r = QRawFont()
+        r.loadFromFile(f, 11.0, QFont.HintingPreference.PreferDefaultHinting)
+        if r.isValid():
+            families.add(r.familyName())
+        else:
             error_dialog(parent, _('Corrupt font'),
-                    _('Failed to read metadata from the font file: %s')%
-                    f, det_msg=traceback.format_exc(), show=True)
+                    _('Failed to load font from the file: {}').format(f), show=True)
             return
-        families.add(fm.font_family)
     families = sorted(families)
 
     dest = os.path.join(config_dir, 'fonts')
@@ -91,7 +113,7 @@ class FontFamilyDelegate(QStyledItemDelegate):
     def sizeHint(self, option, index):
         try:
             return self.do_size_hint(option, index)
-        except:
+        except Exception:
             return QSize(300, 50)
 
     def do_size_hint(self, option, index):
@@ -106,7 +128,7 @@ class FontFamilyDelegate(QStyledItemDelegate):
         painter.save()
         try:
             self.do_paint(painter, option, index)
-        except:
+        except Exception:
             import traceback
             traceback.print_exc()
         painter.restore()
@@ -138,7 +160,7 @@ class FontFamilyDelegate(QStyledItemDelegate):
         painter.drawText(r, Qt.AlignmentFlag.AlignVCenter|Qt.AlignmentFlag.AlignLeading|Qt.TextFlag.TextSingleLine, text)
 
         if (system != QFontDatabase.WritingSystem.Any):
-            w = painter.fontMetrics().horizontalAdvance(text + "  ")
+            w = painter.fontMetrics().horizontalAdvance(text + '  ')
             painter.setFont(font2)
             sample = QFontDatabase.writingSystemSample(system)
             if (option.direction == Qt.LayoutDirection.RightToLeft):
@@ -286,7 +308,7 @@ class FontFamilyDialog(QDialog):
     def build_font_list(self):
         try:
             self.families = list(self.font_scanner.find_font_families())
-        except:
+        except Exception:
             self.families = []
             print('WARNING: Could not load fonts')
             import traceback
