@@ -4,18 +4,25 @@
 import copy
 import json
 import traceback
+
 from qt.core import QDialog, QDialogButtonBox
 
-from calibre.gui2 import error_dialog, gprefs, question_dialog, warning_dialog
+from calibre.gui2 import choose_files, choose_save_file, error_dialog, gprefs, question_dialog, warning_dialog
+from calibre.gui2.dialogs.ff_doc_editor import FFDocEditor
 from calibre.gui2.dialogs.template_dialog import TemplateDialog
 from calibre.gui2.preferences import AbortInitialize, ConfigWidgetBase, test_widget
 from calibre.gui2.preferences.template_functions_ui import Ui_Form
 from calibre.gui2.widgets import PythonHighlighter
 from calibre.utils.formatter_functions import (
-    compile_user_function, compile_user_template_functions, formatter_functions,
-    function_object_type, function_pref_name, load_user_template_functions,
-    StoredObjectType
+    StoredObjectType,
+    compile_user_function,
+    compile_user_template_functions,
+    formatter_functions,
+    function_object_type,
+    function_pref_name,
+    load_user_template_functions,
 )
+from calibre.utils.resources import get_path as P
 from polyglot.builtins import iteritems
 
 
@@ -87,14 +94,15 @@ class ConfigWidget(ConfigWidgetBase, Ui_Form):
         in template processing. You use a stored template in another template as
         if it were a template function, for example 'some_name(arg1, arg2...)'.</p>
 
-        <p>Stored templates must use either General Program Mode -- they must
-        either begin with the text '{0}' or be {1}. You retrieve arguments
-        passed to a GPM stored template using the '{2}()' template function, as
-        in '{2}(var1, var2, ...)'. The passed arguments are copied to the named
-        variables. Arguments passed to a Python template are in the '{2}'
-        parameter. Arguments are always strings.</p>
+        <p>Stored templates must use General Program Mode or Python Template
+        Mode -- they must begin with the text '{0}' or '{1}'. You retrieve
+        arguments passed to a GPM stored template using the '{2}()' template
+        function, as in '{2}(var1, var2, ...)'. The passed arguments are copied
+        to the named variables. Arguments passed to a Python template are in the
+        '{2}' attribute (a list) of the '{3}' parameter. Arguments are always
+        strings.</p>
 
-        <p>For example, this stored template checks if any items are in a
+        <p>For example, this stored GPM template checks if any items are in a
         list, returning '1' if any are found and '' if not.</p>
         <p>
         Template name: items_in_list<br>
@@ -114,7 +122,7 @@ class ConfigWidget(ConfigWidgetBase, Ui_Form):
         See the template language tutorial for more information.</p>
         </p>
         ''')
-        self.st_textBrowser.setHtml(help_text.format('program:', 'python templates', 'arguments'))
+        self.st_textBrowser.setHtml(help_text.format('program:', 'python:', 'arguments', 'context'))
         self.st_textBrowser.adjustSize()
         self.st_show_hide_help_button.clicked.connect(self.st_show_hide_help)
         self.st_textBrowser_height = self.st_textBrowser.height()
@@ -147,7 +155,7 @@ class ConfigWidget(ConfigWidgetBase, Ui_Form):
         try:
             self.builtin_source_dict = json.loads(P('template-functions.json', data=True,
                 allow_user_override=False).decode('utf-8'))
-        except:
+        except Exception:
             traceback.print_exc()
             self.builtin_source_dict = {}
 
@@ -161,7 +169,7 @@ class ConfigWidget(ConfigWidgetBase, Ui_Form):
             for v in self.db.prefs.get('user_template_functions', []):
                 if function_object_type(v) is not StoredObjectType.PythonFunction:
                     self.st_funcs.update({function_pref_name(v):compile_user_function(*v)})
-        except:
+        except Exception:
             if question_dialog(self, _('Template functions'),
                     _('The template functions saved in the library are corrupt. '
                       "Do you want to delete them? Answering 'Yes' will delete all "
@@ -180,6 +188,7 @@ class ConfigWidget(ConfigWidgetBase, Ui_Form):
         self.program.textChanged.connect(self.enable_replace_button)
         self.create_button.clicked.connect(self.create_button_clicked)
         self.delete_button.clicked.connect(self.delete_button_clicked)
+        self.doc_edit_button.clicked.connect(self.doc_edit_button_clicked)
         self.create_button.setEnabled(False)
         self.delete_button.setEnabled(False)
         self.replace_button.setEnabled(False)
@@ -195,20 +204,27 @@ class ConfigWidget(ConfigWidgetBase, Ui_Form):
         self.te_name.editTextChanged.connect(self.st_template_name_edited)
         self.st_create_button.clicked.connect(self.st_create_button_clicked)
         self.st_delete_button.clicked.connect(self.st_delete_button_clicked)
+        self.st_import_button.clicked.connect(self.st_import_button_clicked)
+        self.st_export_button.clicked.connect(self.st_export_button_clicked)
         self.st_create_button.setEnabled(False)
         self.st_delete_button.setEnabled(False)
         self.st_replace_button.setEnabled(False)
         self.st_test_template_button.setEnabled(False)
+        self.st_doc_edit_button.setEnabled(False)
         self.st_clear_button.clicked.connect(self.st_clear_button_clicked)
         self.st_test_template_button.clicked.connect(self.st_test_template)
         self.st_replace_button.clicked.connect(self.st_replace_button_clicked)
+        self.st_doc_edit_button.clicked.connect(self.st_doc_edit_button_clicked)
 
         self.st_current_program_name = ''
         self.st_current_program_text = ''
         self.st_previous_text = ''
         self.st_first_time = False
 
-        self.st_button_layout.insertSpacing(0, 90)
+        # Attempt to properly align the buttons with the template edit widget
+        self.st_button_layout.insertSpacing(0, 70)
+        self.st_button_layout.insertSpacing(self.st_button_layout.indexOf(self.st_doc_edit_button), 60)
+        self.st_button_layout.insertSpacing(self.st_button_layout.indexOf(self.st_test_template_button), 50)
         self.template_editor.new_doc.setFixedHeight(50)
 
         # get field metadata and selected books
@@ -232,6 +248,12 @@ class ConfigWidget(ConfigWidgetBase, Ui_Form):
     def enable_replace_button(self):
         self.replace_button.setEnabled(self.delete_button.isEnabled())
 
+    def doc_edit_button_clicked(self):
+        d = FFDocEditor(can_copy_back=True, parent=self)
+        d.set_document_text(self.documentation.toPlainText())
+        if d.exec() == QDialog.DialogCode.Accepted:
+            self.documentation.setPlainText(d.document_text())
+
     def clear_button_clicked(self):
         self.build_function_names_box()
         self.program.clear()
@@ -301,7 +323,7 @@ class ConfigWidget(ConfigWidgetBase, Ui_Form):
             return True
         if self.argument_count.value() == 0:
             if not question_dialog(self.gui, _('Template functions'),
-                         _('Setting argument count to to zero means that this '
+                         _('Setting argument count to zero means that this '
                            'function cannot be used in single function mode. '
                            'Is this OK?'),
                          det_msg='',
@@ -317,7 +339,7 @@ class ConfigWidget(ConfigWidgetBase, Ui_Form):
             prog = str(self.program.toPlainText())
             compile_user_function(name, str(self.documentation.toPlainText()),
                                         self.argument_count.value(), prog)
-        except:
+        except Exception:
             error_dialog(self.gui, _('Template functions'),
                          _('Exception while compiling function'), show=True,
                          det_msg=traceback.format_exc())
@@ -336,7 +358,7 @@ class ConfigWidget(ConfigWidgetBase, Ui_Form):
                                         self.argument_count.value(), prog)
             self.funcs[name] = cls
             self.build_function_names_box(scroll_to=name)
-        except:
+        except Exception:
             error_dialog(self.gui, _('Template functions'),
                          _('Exception while compiling function'), show=True,
                          det_msg=traceback.format_exc())
@@ -418,6 +440,9 @@ class ConfigWidget(ConfigWidgetBase, Ui_Form):
         self.template_editor.new_doc.clear()
         self.st_create_button.setEnabled(False)
         self.st_delete_button.setEnabled(False)
+        self.st_doc_edit_button.setEnabled(False)
+        self.st_replace_button.setEnabled(False)
+        self.st_current_program_name = ''
 
     def st_build_function_names_box(self, scroll_to=''):
         self.te_name.blockSignals(True)
@@ -433,6 +458,56 @@ class ConfigWidget(ConfigWidgetBase, Ui_Form):
             if idx >= 0:
                 self.te_name.setCurrentIndex(idx)
 
+    def st_import_button_clicked(self):
+        if self.st_replace_button.isEnabled():
+            error_dialog(self, _('Import stored template'),
+                         _('You are currently editing a stored template. Save or clear it'), show=True)
+            return
+        filename = choose_files(self, 'st_import_export_stored_template',
+                _('Import template from file'),
+                filters=[(_('Saved stored template'), ['txt'])],
+                select_only_single_file=True)
+        if filename:
+            self.st_clear_button_clicked()
+            try:
+                with open(filename[0]) as f:
+                    fields = json.load(f)
+                    name = fields['name']
+                    if name in self.st_funcs:
+                        if not question_dialog(self, _('Import stored template'),
+                                               _('A template with the name "{}" already exists. '
+                                                 'Do you want to overwrite it?').format(name),
+                                               show_copy_button=False):
+                            return
+                self.te_name.setCurrentText(name)
+                self.te_textbox.setPlainText(fields['template'])
+                self.template_editor.new_doc.setPlainText(fields['doc'])
+            except Exception as err:
+                traceback.print_exc()
+                error_dialog(self, _('Import template'),
+                             _('<p>Could not import the template. Error:<br>%s')%err, show=True)
+
+    def st_export_button_clicked(self):
+        if not self.te_name.currentText() or not self.te_textbox.toPlainText():
+            error_dialog(self, _('Export stored template'),
+                         _('No template has been selected for export'), show_copy_button=False, show=True)
+            return
+        filename = choose_save_file(self, 'st_import_export_stored_template',
+                _('Export template to file'),
+                filters=[(_('Saved stored template'), ['txt'])],
+                initial_filename=self.te_name.currentText())
+        if filename:
+            try:
+                with open(filename, 'w') as f:
+                    json.dump({'name': self.te_name.currentText(),
+                               'template': self.te_textbox.toPlainText(),
+                               'doc': self.template_editor.new_doc.toPlainText()},
+                               f, indent=1)
+            except Exception as err:
+                traceback.print_exc()
+                error_dialog(self, _('Export template'),
+                             _('<p>Could not export the template. Error:<br>%s')%err, show=True)
+
     def st_delete_button_clicked(self):
         name = str(self.te_name.currentText())
         if name in self.st_funcs:
@@ -440,6 +515,7 @@ class ConfigWidget(ConfigWidgetBase, Ui_Form):
             self.changed_signal.emit()
             self.st_create_button.setEnabled(True)
             self.st_delete_button.setEnabled(False)
+            self.st_doc_edit_button.setEnabled(False)
             self.st_build_function_names_box()
             self.te_textbox.setReadOnly(False)
             self.st_current_program_name = ''
@@ -464,7 +540,7 @@ class ConfigWidget(ConfigWidgetBase, Ui_Form):
                                         0, prog)
             self.st_funcs[name] = cls
             self.st_build_function_names_box(scroll_to=name)
-        except:
+        except Exception:
             error_dialog(self.gui, _('Stored templates'),
                          _('Exception while storing template'), show=True,
                          det_msg=traceback.format_exc())
@@ -476,6 +552,7 @@ class ConfigWidget(ConfigWidgetBase, Ui_Form):
         self.st_delete_button.setEnabled(b)
         self.st_test_template_button.setEnabled(b)
         self.te_textbox.setReadOnly(False)
+        self.st_doc_edit_button.setEnabled(True)
 
     def st_function_index_changed(self, idx):
         txt = self.te_name.currentText()
@@ -511,6 +588,12 @@ class ConfigWidget(ConfigWidgetBase, Ui_Form):
         self.st_delete_button_clicked()
         self.st_create_button_clicked(use_name=name)
 
+    def st_doc_edit_button_clicked(self):
+        d = FFDocEditor(can_copy_back=True, parent=self)
+        d.set_document_text(self.template_editor.new_doc.toPlainText())
+        if d.exec() == QDialog.DialogCode.Accepted:
+            self.template_editor.new_doc.setPlainText(d.document_text())
+
     def commit(self):
         pref_value = []
         for name, cls in iteritems(self.funcs):
