@@ -8,7 +8,6 @@ __docformat__ = 'restructuredtext en'
 
 '''The main GUI'''
 
-import apsw
 import errno
 import gc
 import os
@@ -18,30 +17,37 @@ import textwrap
 import time
 from collections import OrderedDict, deque
 from io import BytesIO
-from qt.core import (
-    QAction, QApplication, QDialog, QFont, QIcon, QMenu, QSystemTrayIcon, Qt, QTimer,
-    QUrl, pyqtSignal,
-)
+
+import apsw
+from qt.core import QAction, QApplication, QDialog, QEvent, QFont, QIcon, QMenu, QSystemTrayIcon, Qt, QTimer, QUrl, pyqtSignal
 
 from calibre import detect_ncpus, force_unicode, prints
-from calibre.constants import (
-    DEBUG, __appname__, config_dir, filesystem_encoding, ismacos, iswindows,
-)
+from calibre.constants import DEBUG, __appname__, config_dir, filesystem_encoding, ismacos, iswindows
 from calibre.customize import PluginInstallationType
 from calibre.customize.ui import available_store_plugins, interface_actions
 from calibre.db.legacy import LibraryDatabase
-from calibre.gui2.extra_files_watcher import ExtraFilesWatcher
 from calibre.gui2 import (
-    Dispatcher, GetMetadata, config, error_dialog, gprefs, info_dialog,
-    max_available_height, open_url, question_dialog, warning_dialog,
+    Dispatcher,
+    GetMetadata,
+    config,
+    error_dialog,
+    gprefs,
+    info_dialog,
+    max_available_height,
+    open_url,
+    question_dialog,
+    timed_print,
+    warning_dialog,
 )
 from calibre.gui2.auto_add import AutoAdder
 from calibre.gui2.changes import handle_changes
 from calibre.gui2.cover_flow import CoverFlowMixin
 from calibre.gui2.device import DeviceMixin
+from calibre.gui2.dialogs.ff_doc_editor import FFDocEditor
 from calibre.gui2.dialogs.message_box import JobError
 from calibre.gui2.ebook_download import EbookDownloadMixin
 from calibre.gui2.email import EmailMixin
+from calibre.gui2.extra_files_watcher import ExtraFilesWatcher
 from calibre.gui2.init import LayoutMixin, LibraryViewMixin
 from calibre.gui2.job_indicator import Pointer
 from calibre.gui2.jobs import JobManager, JobsButton, JobsDialog
@@ -55,12 +61,13 @@ from calibre.gui2.search_box import SavedSearchBoxMixin, SearchBoxMixin
 from calibre.gui2.search_restriction_mixin import SearchRestrictionMixin
 from calibre.gui2.tag_browser.ui import TagBrowserMixin
 from calibre.gui2.update import UpdateMixin
-from calibre.gui2.widgets import ProgressIndicator
+from calibre.gui2.widgets import BusyCursor, ProgressIndicator
 from calibre.library import current_library_name
 from calibre.srv.library_broker import GuiLibraryBroker, db_matches
 from calibre.utils.config import dynamic, prefs
 from calibre.utils.ipc.pool import Pool
-from calibre.utils.resources import get_image_path as I, get_path as P
+from calibre.utils.resources import get_image_path as I
+from calibre.utils.resources import get_path as P
 from polyglot.builtins import string_or_bytes
 from polyglot.queue import Empty, Queue
 
@@ -79,7 +86,7 @@ def add_quick_start_guide(library_view, refresh_cover_browser=None):
     gprefs['quick_start_guide_added'] = True
     imgbuf = BytesIO(calibre_cover2(_('Quick Start Guide'), ''))
     try:
-        with open(P('quick_start/%s.epub' % l), 'rb') as src:
+        with open(P(f'quick_start/{l}.epub'), 'rb') as src:
             buf = BytesIO(src.read())
     except OSError as err:
         if err.errno != errno.ENOENT:
@@ -105,7 +112,6 @@ class Main(MainWindow, MainWindowMixin, DeviceMixin, EmailMixin,  # {{{
         SavedSearchBoxMixin, SearchRestrictionMixin, LayoutMixin, UpdateMixin,
         EbookDownloadMixin
         ):
-
     'The main GUI'
 
     proceed_requested = pyqtSignal(object, object)
@@ -118,7 +124,9 @@ class Main(MainWindow, MainWindowMixin, DeviceMixin, EmailMixin,  # {{{
 
     def __init__(self, opts, parent=None, gui_debug=None):
         MainWindow.__init__(self, opts, parent=parent, disable_automatic_gc=True)
-        self.setWindowIcon(QApplication.instance().windowIcon())
+        self.setVisible(False)
+        if not ismacos:
+            self.setWindowIcon(QApplication.instance().windowIcon())
         self.extra_files_watcher = ExtraFilesWatcher(self)
         self.jobs_pointer = Pointer(self)
         self.proceed_requested.connect(self.do_proceed,
@@ -180,7 +188,7 @@ class Main(MainWindow, MainWindowMixin, DeviceMixin, EmailMixin,  # {{{
             try:
                 st = self.init_istore(store)
                 self.add_istore(st)
-            except:
+            except Exception:
                 # Ignore errors in loading user supplied plugins
                 import traceback
                 traceback.print_exc()
@@ -280,6 +288,7 @@ class Main(MainWindow, MainWindowMixin, DeviceMixin, EmailMixin,  # {{{
                 QIcon.ic('eject.png'), _('&Eject connected device'))
         self.eject_action.setEnabled(False)
         self.addAction(self.quit_action)
+        self.system_tray_menu.addAction(self.iactions['Restart'].menuless_qaction)
         self.system_tray_menu.addAction(self.quit_action)
         self.keyboard.register_shortcut('quit calibre', _('Quit calibre'),
                 default_keys=('Ctrl+Q',), action=self.quit_action)
@@ -322,6 +331,13 @@ class Main(MainWindow, MainWindowMixin, DeviceMixin, EmailMixin,  # {{{
                 action=self.alt_esc_action)
         self.alt_esc_action.triggered.connect(self.clear_additional_restriction)
 
+        self.ff_doc_editor_action = QAction(self)
+        self.addAction(self.ff_doc_editor_action)
+        self.keyboard.register_shortcut('open ff document editor',
+                _('Open the template documentation editor'), default_keys=(''),
+                action=self.ff_doc_editor_action)
+        self.ff_doc_editor_action.triggered.connect(self.open_ff_doc_editor)
+
         # ###################### Start spare job server ########################
         QTimer.singleShot(1000, self.create_spare_pool)
 
@@ -348,11 +364,11 @@ class Main(MainWindow, MainWindowMixin, DeviceMixin, EmailMixin,  # {{{
         if not gprefs.get('quick_start_guide_added', False):
             try:
                 add_quick_start_guide(self.library_view)
-            except:
+            except Exception:
                 import traceback
                 traceback.print_exc()
         for view in ('library', 'memory', 'card_a', 'card_b'):
-            v = getattr(self, '%s_view' % view)
+            v = getattr(self, f'{view}_view')
             v.selectionModel().selectionChanged.connect(self.update_status_bar)
             v.model().count_changed_signal.connect(self.update_status_bar)
 
@@ -372,7 +388,7 @@ class Main(MainWindow, MainWindowMixin, DeviceMixin, EmailMixin,  # {{{
 
         # ########################## Cover Flow ################################
 
-        CoverFlowMixin.init_cover_flow_mixin(self)
+        CoverFlowMixin.__init__(self)
 
         self._calculated_available_height = min(max_available_height()-15,
                 self.height())
@@ -383,7 +399,7 @@ class Main(MainWindow, MainWindowMixin, DeviceMixin, EmailMixin,  # {{{
         for ac in self.iactions.values():
             try:
                 ac.gui_layout_complete()
-            except:
+            except Exception:
                 import traceback
                 traceback.print_exc()
                 if ac.installation_type is PluginInstallationType.BUILTIN:
@@ -391,23 +407,29 @@ class Main(MainWindow, MainWindowMixin, DeviceMixin, EmailMixin,  # {{{
 
         if config['autolaunch_server']:
             self.start_content_server()
-
+        do_hide_windows = False
         if self.system_tray_icon is not None and self.system_tray_icon.isVisible() and opts.start_in_tray:
-            QTimer.singleShot(0, self.hide_windows)
+            do_hide_windows = True
             show_gui = False
             setattr(self, '__systray_minimized', True)
-        if show_gui:
-            self.show()
+        if do_hide_windows:
+            self.hide_windows()
+        self.layout_container.relayout()
+        QTimer.singleShot(0, self.post_initialize_actions)
         self.read_settings()
 
         self.finalize_layout()
         self.bars_manager.start_animation()
         self.set_window_title()
 
+        if show_gui:
+            timed_print('GUI main window shown')
+            self.show()
+
         for ac in self.iactions.values():
             try:
                 ac.initialization_complete()
-            except:
+            except Exception:
                 import traceback
                 traceback.print_exc()
                 if ac.installation_type is PluginInstallationType.BUILTIN:
@@ -421,36 +443,22 @@ class Main(MainWindow, MainWindowMixin, DeviceMixin, EmailMixin,  # {{{
 
         self.listener = Listener(parent=self)
         self.listener.message_received.connect(self.message_from_another_instance)
-        QTimer.singleShot(0, self.listener.start_listening)
-
-        # Collect cycles now
-        gc.collect()
 
         QApplication.instance().shutdown_signal_received.connect(self.quit)
         if show_gui and self.gui_debug is not None:
             QTimer.singleShot(10, self.show_gui_debug_msg)
 
         self.iactions['Connect Share'].check_smartdevice_menus()
-        QTimer.singleShot(1, self.start_smartdevice)
         QTimer.singleShot(100, self.update_toggle_to_tray_action)
 
-        # Once the gui is initialized we can restore the quickview state
-        # The same thing will be true for any action-based operation with a
-        # layout button. We need to let a book be selected in the book list
-        # before initializing quickview, so run it after an event loop tick
-        QTimer.singleShot(0, self.start_quickview)
-        # Force repaint of the book details splitter because it otherwise ends
-        # up with the wrong size. I don't know why.
-        QTimer.singleShot(0, self.bd_splitter.repaint)
-
-    def start_quickview(self):
-        from calibre.gui2.actions.show_quickview import get_quickview_action_plugin
-        qv = get_quickview_action_plugin()
-        if qv:
-            if DEBUG:
-                prints('Starting QuickView')
-            qv.qv_button.restore_state()
-        self.save_layout_state()
+    def post_initialize_actions(self):
+        # Various post-initialization actions after an event loop tick
+        if self.layout_container.is_visible.quick_view or self.iactions['Quickview'].needs_show_on_startup():
+            self.iactions['Quickview'].show_on_startup()
+        self.listener.start_listening()
+        self.start_smartdevice()
+        # Collect cycles now
+        gc.collect()
         self.focus_library_view()
 
     def show_gui_debug_msg(self):
@@ -463,6 +471,9 @@ class Main(MainWindow, MainWindowMixin, DeviceMixin, EmailMixin,  # {{{
     def esc(self, *args):
         self.search.clear()
 
+    def open_ff_doc_editor(self):
+        FFDocEditor(False).exec()
+
     def focus_current_view(self):
         view = self.current_view()
         if view is self.library_view:
@@ -481,13 +492,16 @@ class Main(MainWindow, MainWindowMixin, DeviceMixin, EmailMixin,  # {{{
     def start_smartdevice(self):
         message = None
         if self.device_manager.get_option('smartdevice', 'autostart'):
-            try:
-                message = self.device_manager.start_plugin('smartdevice')
-            except:
-                message = 'start smartdevice unknown exception'
-                prints(message)
-                import traceback
-                traceback.print_exc()
+            timed_print('Starting the smartdevice driver')
+            with BusyCursor():
+                try:
+                    message = self.device_manager.start_plugin('smartdevice')
+                    timed_print('Finished starting smartdevice')
+                except Exception as e:
+                    message = str(e)
+                    timed_print(f'Starting smartdevice driver failed: {message}')
+                    import traceback
+                    traceback.print_exc()
         if message:
             if not self.device_manager.is_running('Wireless Devices'):
                 error_dialog(self, _('Problem starting the wireless device'),
@@ -611,6 +625,17 @@ class Main(MainWindow, MainWindowMixin, DeviceMixin, EmailMixin,  # {{{
                 setattr(window, '__systray_minimized', False)
         self.update_toggle_to_tray_action()
 
+    def changeEvent(self, ev):
+        # Handle bug in Qt 6 that causes the window to be shown as blank if it was first
+        # maximized and then closed to system tray, when remote desktop is
+        # reconnected: https://bugreports.qt.io/browse/QTBUG-124177
+        if (
+                iswindows and ev.type() == QEvent.Type.ActivationChange and self.is_minimized_to_tray and self.isMaximized() and
+                self.isActiveWindow() and not self.isVisible()
+        ):
+            QTimer.singleShot(0, self.show_windows)
+        return super().changeEvent(ev)
+
     def test_server(self, *args):
         if self.content_server is not None and \
                 self.content_server.exception is not None:
@@ -740,7 +765,6 @@ class Main(MainWindow, MainWindowMixin, DeviceMixin, EmailMixin,  # {{{
                 item_val = itemx[4:]
             else:
                 prints('Ignoring invalid item hexval', itemx, file=sys.stderr)
-                return
 
             def doit():
                 nonlocal item_id, item_val
@@ -850,21 +874,30 @@ class Main(MainWindow, MainWindowMixin, DeviceMixin, EmailMixin,  # {{{
             try:
                 argv = json.loads(msg[len('launched:'):])
             except ValueError:
-                prints('Failed to decode message from other instance: %r' % msg)
+                prints(f'Failed to decode message from other instance: {msg!r}')
                 if DEBUG:
                     error_dialog(self, 'Invalid message',
                                  'Received an invalid message from other calibre instance.'
                                  ' Do you have multiple versions of calibre installed?',
-                                 det_msg='Invalid msg: %r' % msg, show=True)
+                                 det_msg=f'Invalid msg: {msg!r}', show=True)
                 argv = ()
             if isinstance(argv, (list, tuple)) and len(argv) > 1:
                 self.handle_cli_args(argv[1:])
             self.setWindowState(self.windowState() & ~Qt.WindowState.WindowMinimized|Qt.WindowState.WindowActive)
             self.show_windows()
-            self.raise_()
+            self.raise_and_focus()
             self.activateWindow()
         elif msg.startswith('shutdown:'):
             self.quit(confirm_quit=False)
+        elif msg.startswith('save-annotations:'):
+            from calibre.gui2.viewer.integration import save_annotations_in_gui
+            try:
+                if not save_annotations_in_gui(self.library_broker, msg[len('save-annotations:'):]):
+                    print('Failed to update annotations for book from viewer, book or library not found.', file=sys.stderr)
+            except Exception:
+                import traceback
+                error_dialog(self, _('Failed to update annotations'), _(
+                    'Failed to update annotations in the database for the book being currently viewed.'), det_msg=traceback.format_exc(), show=True)
         elif msg.startswith('bookedited:'):
             parts = msg.split(':')[1:]
             try:
@@ -886,7 +919,7 @@ class Main(MainWindow, MainWindowMixin, DeviceMixin, EmailMixin,  # {{{
             try:
                 data = json.loads(msg[len('web-store:'):])
             except ValueError:
-                prints('Failed to decode message from other instance: %r' % msg)
+                prints(f'Failed to decode message from other instance: {msg!r}')
             path = data['path']
             if data['tags']:
                 before = self.current_db.new_api.all_book_ids()
@@ -899,11 +932,14 @@ class Main(MainWindow, MainWindowMixin, DeviceMixin, EmailMixin,  # {{{
                     tags += list(data['tags'])
                     self.current_db.new_api.set_field('tags', {book_id: tags})
         else:
-            prints('Ignoring unknown message from other instance: %r' % msg[:20])
+            prints(f'Ignoring unknown message from other instance: {msg[:20]!r}')
 
     def current_view(self):
         '''Convenience method that returns the currently visible view '''
-        idx = self.stack.currentIndex()
+        try:
+            idx = self.stack.currentIndex()
+        except AttributeError:
+            return None  # happens during startup
         if idx == 0:
             return self.library_view
         if idx == 1:
@@ -928,7 +964,7 @@ class Main(MainWindow, MainWindowMixin, DeviceMixin, EmailMixin,  # {{{
                 olddb = self.library_view.model().db
                 if copy_structure:
                     default_prefs = dict(olddb.prefs)
-            except:
+            except Exception:
                 olddb = None
             if copy_structure and olddb is not None and default_prefs is not None:
                 default_prefs['field_metadata'] = olddb.new_api.field_metadata.all_metadata()
@@ -948,15 +984,14 @@ class Main(MainWindow, MainWindowMixin, DeviceMixin, EmailMixin,  # {{{
                             det_msg=traceback.format_exc()
                             )
                     if repair:
-                        from calibre.gui2.dialogs.restore_library import (
-                            repair_library_at,
-                        )
+                        from calibre.gui2.dialogs.restore_library import repair_library_at
                         if repair_library_at(newloc, parent=self):
                             db = LibraryDatabase(newloc, default_prefs=default_prefs)
                         else:
                             return
                     else:
                         return
+            self._save_tb_state(gprefs)
             for action in self.iactions.values():
                 try:
                     action.library_about_to_change(olddb, db)
@@ -983,6 +1018,7 @@ class Main(MainWindow, MainWindowMixin, DeviceMixin, EmailMixin,  # {{{
             if db.new_api.pref('virtual_lib_on_startup'):
                 self.apply_virtual_library(db.new_api.pref('virtual_lib_on_startup'))
             self.rebuild_vl_tabs()
+            self._restore_tb_expansion_state()  # Do this before plugins library_changed()
             for action in self.iactions.values():
                 try:
                     action.library_changed(db)
@@ -1025,9 +1061,9 @@ class Main(MainWindow, MainWindowMixin, DeviceMixin, EmailMixin,  # {{{
         page = 0 if location == 'library' else 1 if location == 'main' else 2 if location == 'carda' else 3
         self.stack.setCurrentIndex(page)
         self.book_details.reset_info()
-        for x in ('tb', 'cb'):
-            splitter = getattr(self, x+'_splitter')
-            splitter.button.setEnabled(location == 'library')
+        self.layout_container.tag_browser_button.setEnabled(location == 'library')
+        self.layout_container.cover_browser_button.setEnabled(location == 'library')
+        self.vl_tabs.update_visibility()
         for action in self.iactions.values():
             action.location_selected(location)
         if location == 'library':
@@ -1106,7 +1142,7 @@ class Main(MainWindow, MainWindowMixin, DeviceMixin, EmailMixin,  # {{{
                     msg = msg[msg.find('calibre.web.feeds.input.RecipeDisabled:'):]
                     msg = msg.partition(':')[-1]
                     d = error_dialog(self, _('Recipe Disabled'),
-                        '<p>%s</p>'%msg)
+                        f'<p>{msg}</p>')
                     d.setModal(False)
                     d.show()
                     self._modeless_dialogs.append(d)
@@ -1122,35 +1158,51 @@ class Main(MainWindow, MainWindowMixin, DeviceMixin, EmailMixin,  # {{{
                             'error':error_dialog}.get(payload['level'],
                                     error_dialog)
                     d = d(self, payload['title'],
-                            '<p>%s</p>'%payload['msg'],
+                            '<p>{}</p>'.format(payload['msg']),
                             det_msg=payload['det_msg'])
                     d.setModal(False)
                     d.show()
                     self._modeless_dialogs.append(d)
                 return
-        except:
+        except Exception:
             pass
         if job.killed:
             return
         try:
             prints(job.details, file=sys.stderr)
-        except:
+        except Exception:
             pass
         if not minz:
             self.job_error_dialog.show_error(dialog_title,
                     _('<b>Failed</b>')+': '+str(job.description),
                     det_msg=job.details, retry_func=retry_func)
 
+    def _save_tb_state(self, gprefs):
+        self.tb_widget.save_state(gprefs)
+        if gprefs['tag_browser_restore_tree_expansion'] and self.current_db is not None:
+            tv_saved_expansions = gprefs.get('tags_view_saved_expansions', {})
+            tv_saved_expansions.update({self.current_db.library_id: self.tb_widget.get_expansion_state()})
+            gprefs['tags_view_saved_expansions'] = tv_saved_expansions
+
+    def _restore_tb_expansion_state(self):
+        if gprefs['tag_browser_restore_tree_expansion'] and self.current_db is not None:
+            tv_saved_expansions = gprefs.get('tags_view_saved_expansions', {})
+            self.tb_widget.restore_expansion_state(tv_saved_expansions.get(self.current_db.library_id))
+
     def read_settings(self):
         self.restore_geometry(gprefs, 'calibre_main_window_geometry', get_legacy_saved_geometry=lambda: config['main_window_geometry'])
         self.read_layout_settings()
+        self._restore_tb_expansion_state()
 
     def write_settings(self):
         with gprefs:  # Only write to gprefs once
             self.save_geometry(gprefs, 'calibre_main_window_geometry')
             dynamic.set('sort_history', self.library_view.model().sort_history)
             self.save_layout_state()
-            self.stack.tb_widget.save_state()
+            self._save_tb_state(gprefs)
+
+    def restart(self):
+        self.quit(restart=True)
 
     def quit(self, checked=True, restart=False, debug_on_restart=False,
             confirm_quit=True, no_plugins_on_restart=False):
@@ -1158,12 +1210,12 @@ class Main(MainWindow, MainWindowMixin, DeviceMixin, EmailMixin,  # {{{
             return
         if confirm_quit and not self.confirm_quit():
             return
+        self.restart_after_quit = restart
         try:
             self.shutdown()
-        except:
+        except Exception:
             import traceback
             traceback.print_exc()
-        self.restart_after_quit = restart
         self.debug_on_restart = debug_on_restart
         self.no_plugins_on_restart = no_plugins_on_restart
         if self.system_tray_icon is not None and self.restart_after_quit:
@@ -1195,7 +1247,10 @@ class Main(MainWindow, MainWindowMixin, DeviceMixin, EmailMixin,  # {{{
         return True
 
     def shutdown(self, write_settings=True):
+        timed_print('Shutdown starting...')
         self.shutting_down = True
+        if hasattr(self.library_view, 'connect_to_book_display_timer'):
+            self.library_view.connect_to_book_display_timer.stop()
         self.shutdown_started.emit()
         self.show_shutdown_message()
         self.server_change_notification_timer.stop()
@@ -1215,7 +1270,7 @@ class Main(MainWindow, MainWindowMixin, DeviceMixin, EmailMixin,  # {{{
         try:
             db = self.library_view.model().db
             cf = db.clean
-        except:
+        except Exception:
             pass
         else:
             cf()
@@ -1243,7 +1298,7 @@ class Main(MainWindow, MainWindowMixin, DeviceMixin, EmailMixin,  # {{{
         def eh(t, v, tb):
             try:
                 traceback.print_exception(t, v, tb, file=sys.stderr)
-            except:
+            except Exception:
                 pass
         sys.excepthook = eh
 
@@ -1254,20 +1309,19 @@ class Main(MainWindow, MainWindowMixin, DeviceMixin, EmailMixin,  # {{{
         self.library_view.model().close()
 
         try:
-            try:
-                if self.content_server is not None:
-                    # If the Content server has any sockets being closed then
-                    # this can take quite a long time (minutes). Tell the user that it is
-                    # happening.
-                    self.show_shutdown_message(
-                        _('Shutting down the Content server. This could take a while...'))
-                    s = self.content_server
-                    self.content_server = None
-                    s.exit()
-            except:
-                pass
+            if self.content_server is not None:
+                # If the Content server has any sockets being closed then
+                # this can take quite a long time (minutes). Tell the user that it is
+                # happening.
+                self.show_shutdown_message(
+                    _('Shutting down the Content server. This could take a while...'))
+                s = self.content_server
+                self.content_server = None
+                s.exit()
         except KeyboardInterrupt:
             pass
+        except Exception:
+            pass
         self.hide_windows()
         if self._spare_pool is not None:
             self._spare_pool.shutdown()
@@ -1280,6 +1334,11 @@ class Main(MainWindow, MainWindowMixin, DeviceMixin, EmailMixin,  # {{{
         wait_for_cleanup()
         wait_for_stop()
         self.shutdown_completed.emit()
+        timed_print('Shutdown complete, quitting...')
+        try:
+            sys.stdout.flush()  # Make sure any buffered prints are written for debug mode
+        except Exception:
+            pass
         return True
 
     def run_wizard(self, *args):
@@ -1288,7 +1347,7 @@ class Main(MainWindow, MainWindowMixin, DeviceMixin, EmailMixin,  # {{{
             self.restart_after_quit = True
             try:
                 self.shutdown(write_settings=False)
-            except:
+            except Exception:
                 pass
             QApplication.instance().quit()
 
@@ -1309,7 +1368,7 @@ class Main(MainWindow, MainWindowMixin, DeviceMixin, EmailMixin,  # {{{
             if self.confirm_quit():
                 try:
                     self.shutdown(write_settings=False)
-                except:
+                except Exception:
                     import traceback
                     traceback.print_exc()
                 e.accept()
