@@ -9,9 +9,10 @@ import calendar
 import json
 import os
 import zipfile
+from collections.abc import Sequence
 from datetime import timedelta
 from threading import RLock
-from typing import Dict, NamedTuple, Optional, Sequence
+from typing import NamedTuple
 
 from lxml import etree
 from lxml.builder import ElementMaker
@@ -57,7 +58,6 @@ def normalize_language(x: str) -> str:
 def serialize_recipe(urn, recipe_class):
     from xml.sax.saxutils import quoteattr
 
-
     def attr(n, d, normalize=lambda x: x):
         ans = getattr(recipe_class, n, d)
         if isinstance(ans, bytes):
@@ -75,15 +75,15 @@ def serialize_recipe(urn, recipe_class):
     if rso:
         options = f' options={quoteattr(json.dumps(rso))}'
     return ('  <recipe id={id} title={title} author={author} language={language}'
-            ' needs_subscription={needs_subscription} description={description}{options}/>').format(**{
-        'id'                 : quoteattr(str(urn)),
-        'title'              : attr('title', _('Unknown')),
-        'author'             : attr('__author__', default_author),
-        'language'           : attr('language', 'und', normalize_language),
-        'needs_subscription' : quoteattr(ns),
-        'description'        : attr('description', ''),
-        'options'            : options,
-        })
+            ' needs_subscription={needs_subscription} description={description}{options}/>').format(
+                id=quoteattr(str(urn)),
+                title=attr('title', _('Unknown')),
+                author=attr('__author__', default_author),
+                language=attr('language', 'und', normalize_language),
+                needs_subscription=quoteattr(ns),
+                description=attr('description', ''),
+                options=options,
+            )
 
 
 def serialize_collection(mapping_of_recipe_classes):
@@ -94,7 +94,7 @@ def serialize_collection(mapping_of_recipe_classes):
                 'utf-8')):
         try:
             recipe = serialize_recipe(urn, mapping_of_recipe_classes[urn])
-        except:
+        except Exception:
             import traceback
             traceback.print_exc()
             continue
@@ -113,8 +113,8 @@ def serialize_builtin_recipes():
         with open(f, 'rb') as stream:
             try:
                 recipe_class = compile_recipe(stream.read())
-            except:
-                print('Failed to compile: %s'%f)
+            except Exception:
+                print(f'Failed to compile: {f}')
                 raise
         if recipe_class is not None:
             recipe_mapping['builtin:'+rid] = recipe_class
@@ -138,9 +138,9 @@ def get_custom_recipe_collection(*args):
                 recipe = f.read().decode('utf-8')
             recipe_class = compile_recipe(recipe)
             if recipe_class is not None:
-                rmap['custom:%s'%id_] = recipe_class
-        except:
-            print('Failed to load recipe from: %r'%fname)
+                rmap[f'custom:{id_}'] = recipe_class
+        except Exception:
+            print(f'Failed to load recipe from: {fname!r}')
             import traceback
             traceback.print_exc()
             continue
@@ -215,7 +215,7 @@ def remove_custom_recipe(id_):
         del custom_recipes[id_]
         try:
             delete_file(os.path.join(bdir, fname))
-        except:
+        except Exception:
             pass
 
 
@@ -265,7 +265,7 @@ def get_builtin_recipe_by_title(title, log=None, download_recipe=False):
                     if log is not None:
                         log('Trying to get latest version of recipe:', urn)
                     return download_builtin_recipe(urn)
-                except:
+                except Exception:
                     if log is None:
                         import traceback
                         traceback.print_exc()
@@ -284,7 +284,7 @@ def get_builtin_recipe_by_id(id_, log=None, download_recipe=False):
                     if log is not None:
                         log('Trying to get latest version of recipe:', urn)
                     return download_builtin_recipe(urn)
-                except:
+                except Exception:
                     if log is None:
                         import traceback
                         traceback.print_exc()
@@ -298,7 +298,7 @@ class RecipeCustomization(NamedTuple):
     add_title_tag: bool = False
     custom_tags: Sequence[str] = ()
     keep_issues: int = 0
-    recipe_specific_options: Optional[Dict[str, str]] = None
+    recipe_specific_options: dict[str, str] | None = None
 
 
 class SchedulerConfig:
@@ -314,7 +314,7 @@ class SchedulerConfig:
             with ExclusiveFile(self.conf_path) as f:
                 try:
                     self.root = safe_xml_fromstring(f.read(), recover=False)
-                except:
+                except Exception:
                     print('Failed to read recipe scheduler config')
                     import traceback
                     traceback.print_exc()
@@ -323,17 +323,17 @@ class SchedulerConfig:
 
     def iter_recipes(self):
         for x in self.root:
-            if x.tag == '{%s}scheduled_recipe'%NS:
+            if x.tag == f'{{{NS}}}scheduled_recipe':
                 yield x
 
     def iter_accounts(self):
         for x in self.root:
-            if x.tag == '{%s}account_info'%NS:
+            if x.tag == f'{{{NS}}}account_info':
                 yield x
 
     def iter_customization(self):
         for x in self.root:
-            if x.tag == '{%s}recipe_customization'%NS:
+            if x.tag == f'{{{NS}}}recipe_customization':
                 yield x
 
     def schedule_recipe(self, recipe, schedule_type, schedule, last_downloaded=None):
@@ -351,7 +351,7 @@ class SchedulerConfig:
             if last_downloaded is None:
                 last_downloaded = EPOCH
             sr = E.scheduled_recipe({
-                'id' : recipe.get('id'),
+                'id': recipe.get('id'),
                 'title': recipe.get('title'),
                 'last_downloaded':isoformat(last_downloaded),
                 }, self.serialize_schedule(schedule_type, schedule))
@@ -365,10 +365,10 @@ class SchedulerConfig:
                 if x.get('id') == urn:
                     self.root.remove(x)
             cs = E.recipe_customization({
-                'keep_issues' : str(val.keep_issues),
-                'id' : urn,
-                'add_title_tag' : 'yes' if val.add_title_tag else 'no',
-                'custom_tags' : ','.join(val.custom_tags),
+                'keep_issues': str(val.keep_issues),
+                'id': urn,
+                'add_title_tag': 'yes' if val.add_title_tag else 'no',
+                'custom_tags': ','.join(val.custom_tags),
                 'recipe_specific_options': json.dumps(val.recipe_specific_options or {}),
                 })
             self.root.append(cs)
@@ -425,14 +425,14 @@ class SchedulerConfig:
         if typ == 'interval':
             if schedule < 0.04:
                 schedule = 0.04
-            text = '%f'%schedule
+            text = f'{schedule:f}'
         elif typ == 'day/time':
-            text = '%d:%d:%d'%schedule
+            text = f'{int(schedule[0])}:{int(schedule[1])}:{int(schedule[2])}'
         elif typ in ('days_of_week', 'days_of_month'):
             dw = ','.join(map(str, map(int, schedule[0])))
-            text = '%s:%d:%d'%(dw, schedule[1], schedule[2])
+            text = f'{dw}:{int(schedule[1])}:{int(schedule[2])}'
         else:
-            raise ValueError('Unknown schedule type: %r'%typ)
+            raise ValueError(f'Unknown schedule type: {typ!r}')
         s.text = text
         return s
 
@@ -458,7 +458,7 @@ class SchedulerConfig:
     def recipe_needs_to_be_downloaded(self, recipe):
         try:
             typ, sch, ld = self.un_serialize_schedule(recipe)
-        except:
+        except Exception:
             return False
 
         def is_time(now, hour, minute):
@@ -565,7 +565,7 @@ class SchedulerConfig:
         for r in c.get('scheduled_recipes', []):
             try:
                 self.add_old_recipe(r)
-            except:
+            except Exception:
                 continue
         for k in c.keys():
             if k.startswith('recipe_account_info'):
@@ -574,20 +574,20 @@ class SchedulerConfig:
                     if urn.startswith('recipe_'):
                         urn = 'builtin:'+urn[7:]
                     else:
-                        urn = 'custom:%d'%int(urn)
+                        urn = f'custom:{int(urn)}'
                     try:
                         username, password = c[k]
-                    except:
+                    except Exception:
                         username = password = ''
                     self.set_account_info(urn, str(username),
                             str(password))
-                except:
+                except Exception:
                     continue
         del c
         self.write_scheduler_file()
         try:
             os.remove(old_conf_path)
-        except:
+        except Exception:
             pass
 
     def add_old_recipe(self, r):
@@ -596,14 +596,14 @@ class SchedulerConfig:
             urn = 'builtin:'+r['id'][7:]
         elif not r['builtin']:
             try:
-                urn = 'custom:%d'%int(r['id'])
-            except:
+                urn = 'custom:{}'.format(int(r['id']))
+            except Exception:
                 return
         schedule = r['schedule']
         typ = 'interval'
         if schedule > 1e5:
             typ = 'day/time'
-            raw = '%d'%int(schedule)
+            raw = str(int(schedule))
             day = int(raw[0]) - 1
             hour = int(raw[2:4]) - 1
             minute = int(raw[-2:]) - 1
